=== modified file 'NEWS'
--- NEWS	2007-08-14 19:29:56 +0000
+++ NEWS	2007-08-15 01:12:57 +0000
@@ -2,6 +2,11 @@
 
   BUGFIXES:
 
+  INTERNALS:
+
+    * ``bzrlib.pack.ContainerWriter`` now tracks how many records have been
+      added via a public attribute records_written. (Robert Collins)
+
 bzr 0.90rc1 2007-08-14
 
   BUGFIXES:

=== modified file 'bzrlib/pack.py'
--- bzrlib/pack.py	2007-08-08 02:57:22 +0000
+++ bzrlib/pack.py	2007-08-15 01:12:57 +0000
@@ -59,7 +59,12 @@
 
 
 class ContainerWriter(object):
-    """A class for writing containers."""
+    """A class for writing containers.
+
+    :attribute records_written: The number of user records added to the
+        container. This does not count the prelude or suffix of the container
+        introduced by the begin() and end() methods.
+    """
 
     def __init__(self, write_func):
         """Constructor.
@@ -69,6 +74,7 @@
         """
         self._write_func = write_func
         self.current_offset = 0
+        self.records_written = 0
 
     def begin(self):
         """Begin writing a container."""
@@ -112,6 +118,7 @@
         self.write_func("\n")
         # Finally, the contents.
         self.write_func(bytes)
+        self.records_written += 1
         # return a memo of where we wrote data to allow random access.
         return current_offset, self.current_offset - current_offset
 

=== modified file 'bzrlib/tests/test_pack.py'
--- bzrlib/tests/test_pack.py	2007-08-08 02:57:22 +0000
+++ bzrlib/tests/test_pack.py	2007-08-15 01:12:57 +0000
@@ -40,6 +40,13 @@
         self.assertEqual('Bazaar pack format 1 (introduced in 0.18)\n',
                          output.getvalue())
 
+    def test_zero_records_written_after_begin(self):
+        """After begin is written, 0 records have been written."""
+        output = StringIO()
+        writer = pack.ContainerWriter(output.write)
+        writer.begin()
+        self.assertEqual(0, writer.records_written)
+
     def test_end(self):
         """The end() method writes an End Marker record."""
         output = StringIO()
@@ -49,6 +56,23 @@
         self.assertEqual('Bazaar pack format 1 (introduced in 0.18)\nE',
                          output.getvalue())
 
+    def test_empty_end_does_not_add_a_record_to_records_written(self):
+        """The end() method does not count towards the records written."""
+        output = StringIO()
+        writer = pack.ContainerWriter(output.write)
+        writer.begin()
+        writer.end()
+        self.assertEqual(0, writer.records_written)
+
+    def test_non_empty_end_does_not_add_a_record_to_records_written(self):
+        """The end() method does not count towards the records written."""
+        output = StringIO()
+        writer = pack.ContainerWriter(output.write)
+        writer.begin()
+        writer.add_bytes_record('foo', names=[])
+        writer.end()
+        self.assertEqual(1, writer.records_written)
+
     def test_add_bytes_record_no_name(self):
         """Add a bytes record with no name."""
         output = StringIO()
@@ -131,6 +155,16 @@
             errors.InvalidRecordError,
             writer.add_bytes_record, 'abc', names=[('bad name', )])
 
+    def test_add_bytes_records_add_to_records_written(self):
+        """Adding a Bytes record increments the records_written counter."""
+        output = StringIO()
+        writer = pack.ContainerWriter(output.write)
+        writer.begin()
+        writer.add_bytes_record('foo', names=[])
+        self.assertEqual(1, writer.records_written)
+        writer.add_bytes_record('foo', names=[])
+        self.assertEqual(2, writer.records_written)
+
 
 class TestContainerReader(tests.TestCase):
 

